VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsMP3ID3v1"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' cMP3ID3v1.cls
' Read and Write MP3 ID3 v1 tags.
'
' http://vbaccelerator.com/
' S McMahon
' 2003-01-26
'
' Updates
' 2003/05/07: Added ID3 v1.1 support.
' Thanks to Don Beal for providing the code!
'
' 2004/01/19: Added code to strip null characters
' from the strings returned by the tag properties.


Private Type MP3ID3V1Tag
  Tag As String * 3           '-- 03
  Title As String * 30     '-- 33
  Artist As String * 30       '-- 63
  Album As String * 30        '-- 93
  Year As String * 4          '-- 97
  Comment As String * 28      '-- 125
  ' START: Mods for ID3 v1.1 support
  Filler As Byte              '-- 126
  Track As Byte               '-- 127
  ' END: Mods for ID3 v1.1. support
  Genre As Byte               '-- 128
End Type

Private m_sMp3File As String

Private m_bHasID3v1Tag As Boolean

Private m_sTag As String
Private m_sTitle As String
Private m_sArtist As String
Private m_sAlbum As String
Private m_sYear As String
Private m_sComment As String
Private m_sGenre As Byte
' Added for ID3 v1.1 support:
Private m_sTrack As Byte


Public Property Get MP3File() As String
   MP3File = m_sMp3File
End Property

Public Property Let MP3File(ByVal value As String)
   m_sMp3File = value
   pLoadTag
End Property

Public Sub Update()
   pUpdateTag
End Sub

Public Property Get HasID3v1Tag() As Boolean
   HasID3v1Tag = m_bHasID3v1Tag
End Property

Public Property Get Title() As String
   Title = m_sTitle
End Property
Public Property Let Title(ByVal value As String)
   m_sTitle = value
End Property

Public Property Get Artist() As String
   Artist = m_sArtist
End Property
Public Property Let Artist(ByVal value As String)
   m_sArtist = value
End Property

Public Property Get Album() As String
   Album = m_sAlbum
End Property
Public Property Let Album(ByVal value As String)
   m_sAlbum = value
End Property

Public Property Get Year() As String
   Year = m_sYear
End Property
Public Property Let Year(ByVal value As String)
   m_sYear = value
End Property

' START: Added for ID3 v1.1 support.
Public Property Get Track() As Byte
   Track = m_sTrack
End Property
Public Property Let Track(ByVal value As Byte)
   m_sTrack = value
End Property
' END: Added for ID3 v1.1 support.

Public Property Get Comment() As String
   Comment = m_sComment
End Property
Public Property Let Comment(ByVal value As String)
   m_sComment = value
End Property

Public Property Get Genre() As Byte
   Genre = m_sGenre
End Property
Public Property Let Genre(ByVal value As Byte)
   m_sGenre = value
End Property



Private Sub pLoadTag()
Dim iFile As Integer
Dim lErr As Long
Dim sErr As String
   
   m_bHasID3v1Tag = False
   m_sComment = ""
   m_sArtist = ""
   m_sAlbum = ""
   m_sYear = ""
   m_sGenre = 255
   m_sTitle = ""
   ' Added for ID3 v1.1 support:
   m_sTrack = 0
   
   iFile = FreeFile
   On Error Resume Next
   Open m_sMp3File For Binary Access Read Lock Write As #iFile
   If (Err.Number <> 0) Then
      lErr = Err.Number
      sErr = Err.Description
      On Error GoTo 0
      'Err.Raise lErr, App.EXEName & ".cMP3ID3v1", sErr
   Else
      On Error GoTo 0
      If LOF(iFile) > 128 Then
         Dim Tag As MP3ID3V1Tag
         Get #iFile, LOF(iFile) - 127, Tag.Tag
         If Not (StrComp(Tag.Tag, "TAG") = 0) Then
            '  no tag
         Else
            m_bHasID3v1Tag = True
            Get #iFile, , Tag.Title
            m_sTitle = psStripNulls(Tag.Title)
            Get #iFile, , Tag.Artist
            m_sArtist = psStripNulls(Tag.Artist)
            Get #iFile, , Tag.Album
            m_sAlbum = psStripNulls(Tag.Album)
            Get #iFile, , Tag.Year
            m_sYear = psStripNulls(Tag.Year)
            Get #iFile, , Tag.Comment
            m_sComment = psStripNulls(Tag.Comment)
            ' START: Added for ID3 v1.1 support:
            Get #iFile, , Tag.Filler
            Get #iFile, , Tag.Track
            m_sTrack = psStripNulls(Tag.Track)
            ' END: added for ID3 v1.1 support
            Get #iFile, , Tag.Genre
            m_sGenre = psStripNulls(Tag.Genre)
            
         End If
      End If
   End If
   On Error Resume Next
   Close #iFile
   On Error GoTo 0
   Err.Clear
   
End Sub
Private Function psStripNulls(ByVal sBuf As String) As String
Dim i As Long
Dim sTag As String
   
   sTag = ""
   For i = 1 To Len(sBuf)
      If Not (Asc(Mid(sBuf, i, 1)) = 0) Then
         sTag = sTag & Mid(sBuf, i, 1)
      End If
   Next i
   psStripNulls = Trim(sTag)
   
End Function
Private Sub pUpdateTag()
Dim iFile As Integer
Dim lErr As Long
Dim sErr As String
   iFile = FreeFile
   On Error Resume Next
   Open m_sMp3File For Binary Access Read Write Lock Write As #iFile
   If (Err.Number <> 0) Then
      lErr = Err.Number
      sErr = Err.Description
      On Error GoTo 0
      'Err.Raise lErr, App.EXEName & ".cMP3ID3v1", sErr
   Else
      Dim Tag As MP3ID3V1Tag
      If LOF(iFile) > 0 Then
         If LOF(iFile) > 128 Then
            Get #iFile, LOF(iFile) - 127, Tag.Tag
            If Not (StrComp(Tag.Tag, "TAG") = 0) Then
               ' no MP3 tag already, need to extend the file
               ' to add it
               Seek #iFile, LOF(iFile)
               Tag.Tag = "TAG"
               Put #iFile, , Tag.Tag
            End If
            LSet Tag.Title = m_sTitle
            LSet Tag.Artist = m_sArtist
            LSet Tag.Album = m_sAlbum
            LSet Tag.Year = m_sYear
            ' START: added for ID3 v1.1 support
            Tag.Track = m_sTrack
            ' END: added for ID3 v1.1 support
            LSet Tag.Comment = m_sComment
            Tag.Genre = m_sGenre
            
            Put #iFile, , Tag.Title
            Put #iFile, , Tag.Artist
            Put #iFile, , Tag.Album
            Put #iFile, , Tag.Year
            Put #iFile, , Tag.Comment
            ' START: added for ID3 v1.1 support
            Put #iFile, , Tag.Filler
            Put #iFile, , Tag.Track
            ' END: added for ID3 v1.1 support
            Put #iFile, , Tag.Genre
            
         End If
      Else
         On Error Resume Next
         Close #iFile
         On Error GoTo 0
         Err.Raise vbObjectError + 4097, App.EXEName & ".cMP3ID3v1", m_sMp3File _
          & " is not a valid MP3 file."
      End If
   End If
   
   On Error Resume Next
   Close #iFile
   On Error GoTo 0
   Err.Clear
   
End Sub


